@@ -1,12 +1,13 @@
 # This relies on each of the submodules having an __all__ variable.
 
-__version__ = '0.18.1'
+__version__ = '0.20.0'
 
 
 from . import hdrs  # noqa
 from .protocol import *  # noqa
 from .connector import *  # noqa
 from .client import *  # noqa
+from .client_reqrep import *  # noqa
 from .errors import *  # noqa
 from .helpers import *  # noqa
 from .parsers import *  # noqa
@@ -17,6 +18,7 @@ from .websocket_client import *  # noqa
 
 
 __all__ = (client.__all__ +
+           client_reqrep.__all__ +
            errors.__all__ +
            helpers.__all__ +
            parsers.__all__ +
