@@ -1,4 +1,5 @@
 import asyncio
+import collections
 import http.cookies
 import io
 import json
@@ -23,6 +24,9 @@ from .multidict import (CIMultiDictProxy, MultiDictProxy, MultiDict,
 from .multipart import MultipartWriter
 from .protocol import HttpMessage
 
+
+__all__ = ('ClientRequest', 'ClientResponse')
+
 PY_35 = sys.version_info >= (3, 5)
 
 HTTP_PORT = 80
@@ -161,13 +165,12 @@ class ClientRequest:
         if not path:
             path = '/'
 
-        if isinstance(params, dict):
-            params = list(params.items())
-        elif isinstance(params, (MultiDictProxy, MultiDict)):
+        if isinstance(params, collections.Mapping):
             params = list(params.items())
 
         if params:
-            params = urllib.parse.urlencode(params)
+            if not isinstance(params, str):
+                params = urllib.parse.urlencode(params)
             if query:
                 query = '%s&%s' % (query, params)
             else:
@@ -220,8 +223,7 @@ class ClientRequest:
 
         for name, value in cookies:
             if isinstance(value, http.cookies.Morsel):
-                # use dict method because SimpleCookie class modifies value
-                dict.__setitem__(c, name, value)
+                c[value.key] = value.value
             else:
                 c[name] = value
 
@@ -231,8 +233,10 @@ class ClientRequest:
         """Set request content encoding."""
         enc = self.headers.get(hdrs.CONTENT_ENCODING, '').lower()
         if enc:
-            self.compress = enc
-            self.chunked = True  # enable chunked, no need to deal with length
+            if self.compress is not False:
+                self.compress = enc
+                # enable chunked, no need to deal with length
+                self.chunked = True
         elif self.compress:
             if not isinstance(self.compress, str):
                 self.compress = 'deflate'
@@ -475,7 +479,7 @@ class ClientRequest:
                            for k, value in self.headers.items())))
         request.send_headers()
 
-        self._writer = asyncio.async(
+        self._writer = helpers.ensure_future(
             self.write_bytes(request, reader), loop=self.loop)
 
         self.response = self.response_class(
@@ -534,6 +538,7 @@ class ClientResponse:
         self._continue = continue100
         self._closed = False
         self._should_close = True  # override by message.should_close later
+        self._history = ()
 
     def _post_init(self, loop):
         self._loop = loop
@@ -564,6 +569,11 @@ class ClientResponse:
     def connection(self):
         return self._connection
 
+    @property
+    def history(self):
+        """A sequence of of responses, if redirects occured."""
+        return self._history
+
     def waiting_for_continue(self):
         return self._continue is not None
 
@@ -693,14 +703,6 @@ class ClientResponse:
 
         return data
 
-    @asyncio.coroutine
-    def read_and_close(self, decode=False):
-        """Read response payload and then close response."""
-        warnings.warn(
-            'read_and_close is deprecated, use .read() instead',
-            DeprecationWarning)
-        return (yield from self.read(decode))
-
     def _get_encoding(self):
         ctype = self.headers.get(hdrs.CONTENT_TYPE, '').lower()
         mtype, stype, _, params = helpers.parse_mimetype(ctype)
