@@ -10,9 +10,10 @@ import inspect
 
 from collections.abc import Sized, Iterable, Container
 from urllib.parse import urlencode, unquote
+from types import MappingProxyType
 
 from . import hdrs
-from .abc import AbstractRouter, AbstractMatchInfo
+from .abc import AbstractRouter, AbstractMatchInfo, AbstractView
 from .protocol import HttpVersion11
 from .web_exceptions import HTTPMethodNotAllowed, HTTPNotFound, HTTPNotModified
 from .web_reqrep import StreamResponse
@@ -20,7 +21,7 @@ from .multidict import upstr
 
 
 __all__ = ('UrlDispatcher', 'UrlMappingMatchInfo',
-           'Route', 'PlainRoute', 'DynamicRoute', 'StaticRoute')
+           'Route', 'PlainRoute', 'DynamicRoute', 'StaticRoute', 'View')
 
 
 class UrlMappingMatchInfo(dict, AbstractMatchInfo):
@@ -162,6 +163,9 @@ class StaticRoute(Route):
             raise ValueError(
                 "No directory exists at '{}'".format(self._directory))
 
+        if bool(os.environ.get("AIOHTTP_NOSENDFILE")):
+            self._sendfile = self._sendfile_fallback
+
     def match(self, path):
         if not path.startswith(self._prefix):
             return None
@@ -283,10 +287,15 @@ class StaticRoute(Route):
         file_size = st.st_size
 
         resp.content_length = file_size
-        yield from resp.prepare(request)
+        resp.set_tcp_cork(True)
+        try:
+            yield from resp.prepare(request)
 
-        with open(filepath, 'rb') as f:
-            yield from self._sendfile(request, resp, f, file_size)
+            with open(filepath, 'rb') as f:
+                yield from self._sendfile(request, resp, f, file_size)
+
+        finally:
+            resp.set_tcp_nodelay(True)
 
         return resp
 
@@ -365,6 +374,23 @@ class _MethodNotAllowedMatchInfo(UrlMappingMatchInfo):
                         ', '.join(sorted(self._allowed_methods))))
 
 
+class View(AbstractView):
+
+    @asyncio.coroutine
+    def __iter__(self):
+        if self.request.method not in hdrs.METH_ALL:
+            self._raise_allowed_methods()
+        method = getattr(self, self.request.method.lower(), None)
+        if method is None:
+            self._raise_allowed_methods()
+        resp = yield from method()
+        return resp
+
+    def _raise_allowed_methods(self):
+        allowed_methods = {m for m in hdrs.METH_ALL if hasattr(self, m)}
+        raise HTTPMethodNotAllowed(self.request.method, allowed_methods)
+
+
 class RoutesView(Sized, Iterable, Container):
 
     __slots__ = '_urls'
@@ -389,7 +415,7 @@ class UrlDispatcher(AbstractRouter, collections.abc.Mapping):
         r'^\{(?P<var>[a-zA-Z][_a-zA-Z0-9]*):(?P<re>.+)\}$')
     GOOD = r'[^{}/]+'
     ROUTE_RE = re.compile(r'(\{[_a-zA-Z][^{}]*(?:\{[^{}]*\}[^{}]*)*\})')
-    NAME_SPLIT_RE = re.compile('[.:]')
+    NAME_SPLIT_RE = re.compile('[.:-]')
 
     METHODS = {hdrs.METH_ANY, hdrs.METH_POST,
                hdrs.METH_GET, hdrs.METH_PUT, hdrs.METH_DELETE,
@@ -440,6 +466,9 @@ class UrlDispatcher(AbstractRouter, collections.abc.Mapping):
     def routes(self):
         return RoutesView(self._urls)
 
+    def named_routes(self):
+        return MappingProxyType(self._routes)
+
     def register_route(self, route):
         assert isinstance(route, Route), 'Instance of Route class is required.'
 
@@ -449,10 +478,10 @@ class UrlDispatcher(AbstractRouter, collections.abc.Mapping):
             parts = self.NAME_SPLIT_RE.split(name)
             for part in parts:
                 if not part.isidentifier() or keyword.iskeyword(part):
-                    raise ValueError('Incorrect route name value, '
-                                     'Route name should be a sequence of '
+                    raise ValueError('Incorrect route name {!r}, '
+                                     'the name should be a sequence of '
                                      'python identifiers separated '
-                                     'by dot or column')
+                                     'by dash, dot or column'.format(name))
             if name in self._routes:
                 raise ValueError('Duplicate {!r}, '
                                  'already handled by {!r}'
@@ -467,8 +496,13 @@ class UrlDispatcher(AbstractRouter, collections.abc.Mapping):
             raise ValueError("path should be started with /")
 
         assert callable(handler), handler
-        if (not asyncio.iscoroutinefunction(handler) and
-                not inspect.isgeneratorfunction(handler)):
+        if asyncio.iscoroutinefunction(handler):
+            pass
+        elif inspect.isgeneratorfunction(handler):
+            pass
+        elif isinstance(handler, type) and issubclass(handler, AbstractView):
+            pass
+        else:
             handler = asyncio.coroutine(handler)
 
         method = upstr(method)
