@@ -11,11 +11,10 @@ import inspect
 import io
 import os
 import sys
-import time
 from urllib.parse import urlsplit
 
 import aiohttp
-from aiohttp import server, helpers, hdrs
+from aiohttp import server, hdrs
 
 __all__ = ('WSGIServerHttpProtocol',)
 
@@ -64,17 +63,10 @@ class WSGIServerHttpProtocol(server.ServerHttpProtocol):
             'SERVER_PROTOCOL': 'HTTP/%s.%s' % message.version
         }
 
-        # authors should be aware that REMOTE_HOST and REMOTE_ADDR
-        # may not qualify the remote addr:
-        # http://www.ietf.org/rfc/rfc3875
-        forward = self.transport.get_extra_info('addr', '127.0.0.1')
         script_name = self.SCRIPT_NAME
-        server = forward
 
         for hdr_name, hdr_value in message.headers.items():
-            if hdr_name == 'HOST':
-                server = hdr_value
-            elif hdr_name == 'SCRIPT_NAME':
+            if hdr_name == 'SCRIPT_NAME':
                 script_name = hdr_value
             elif hdr_name == 'CONTENT-TYPE':
                 environ['CONTENT_TYPE'] = hdr_value
@@ -89,17 +81,29 @@ class WSGIServerHttpProtocol(server.ServerHttpProtocol):
 
             environ[key] = hdr_value
 
-        remote = helpers.parse_remote_addr(forward)
-        environ['REMOTE_ADDR'] = remote[0]
-        environ['REMOTE_PORT'] = remote[1]
+        # authors should be aware that REMOTE_HOST and REMOTE_ADDR
+        # may not qualify the remote addr
+        # also SERVER_PORT variable MUST be set to the TCP/IP port number on
+        # which this request is received from the client.
+        # http://www.ietf.org/rfc/rfc3875
 
-        if isinstance(server, str):
-            server = server.split(':')
-            if len(server) == 1:
-                server.append('80' if url_scheme == 'http' else '443')
-
-        environ['SERVER_NAME'] = server[0]
-        environ['SERVER_PORT'] = str(server[1])
+        remote = self.transport.get_extra_info('peername')
+        if remote:
+            environ['REMOTE_ADDR'] = remote[0]
+            environ['REMOTE_PORT'] = remote[1]
+            _host, port = self.transport.get_extra_info('sockname')
+            environ['SERVER_PORT'] = str(port)
+            host = message.headers.get("HOST", None)
+            # SERVER_NAME should be set to value of Host header, but this
+            # header is not required. In this case we shoud set it to local
+            # address of socket
+            environ['SERVER_NAME'] = host.split(":")[0] if host else _host
+        else:
+            # Dealing with unix socket, so request was received from client by
+            # upstream server and this data may be found in the headers
+            for header in ('REMOTE_ADDR', 'REMOTE_PORT',
+                           'SERVER_NAME', 'SERVER_PORT'):
+                environ[header] = message.headers.get(header, '')
 
         path_info = uri_parts.path
         if script_name:
@@ -116,7 +120,7 @@ class WSGIServerHttpProtocol(server.ServerHttpProtocol):
     @asyncio.coroutine
     def handle_request(self, message, payload):
         """Handle a single HTTP request"""
-        now = time.time()
+        now = self._loop.time()
 
         if self.readpayload:
             wsgiinput = io.BytesIO()
@@ -147,7 +151,7 @@ class WSGIServerHttpProtocol(server.ServerHttpProtocol):
             self.keep_alive(True)
 
         self.log_access(
-            message, environ, response.response, time.time() - now)
+            message, environ, response.response, self._loop.time() - now)
 
 
 class FileWrapper:
